/*******************************************************************************
Title:				Testing Interfaces 
Files:				TestSequence.java, NoCurrentException.java, CWHelp.java,
					Sequence.class

Author:				Data Fok, data@cs.wisc.edu
Collaborators:			Kevin Wang, kevinwks@hotmail.com
Due Date:				Tuesday, February 13, 2001
Completion Date:		Tuesday, February 13, 2001

Course:				CS367, Spring 2001, Section 3
Assignment:			Program 1 

Editor:				CodeWarrior
Compiler:			CodeWarrior (JDK 1.2)
Platform:			Unix (Solaris)
*******************************************************************************/

// This is the ruler discussed in the "Style Guide".  You normally don't leave 
// it in a program but it helps make sure you stick to 80 characters or less
// per line.
//345678 112345678 212345678 312345678 412345678 512345678 612345678 712345678 8

import java.io.*;

/**
 * Class TestSequence  Testing whether the given Sequence class is true to the public interface 
 * by different test cases. An output file TS.out will print out the detectedd expection, 
 * throwable and its stack trace. There are four main types of sequence tested here, they are:
 * empty sequence, 1-item sequence, larger number of item sequence and a really big number of
 * items. String and Integer sequence will be consider in those test cases to make sure the ADT 
 * is as general as it claims.
 * Bugs: none.
 **/
 
 /*
 	List of test cases being performed:
	I) Empty sequence
	  I.2) Constructor
	  I.4) See if size works
	  I.6) See if start works.
	  I.8) See if isCurrent false after start
	  I.10) See if back works
	  I.12) See if isCurrent false after back
	  I.14) See if getCurrent throws exception on empty Sequence
	  I.16) See if lookup works, e.g., it isn't found.
	  I.18) See if removeCurrent works
	  I.20) See if advance works
	  I.22) See if isCurrent false after advance
	  
	II) size 1 sequence
	  II.2) Try addAfter to put in the one item.
          II.4) See if isCurrent true after addAfter
          II.6) See if start works after addAfter.
          II.8) See if size works after addAfter.
          II.10) See if lookup works addAfter.
          II.12) See if getCurrent works after addAfter.
          II.14) See if back works after addAfter and start
	  II.16) See if isCurrent false after addAfter and back
	  II.18) See if advance works after addAfter and start
	  II.20) See if isCurrent false after addAfter and advance
	  II.22) See if removeCurrent works on size 1 Sequence.
	  II.24) See if isCurrent false after removeCurrent
	  II.26) See if removeCurrent throws exception when current is undefined
	  II.28) See if size works after removeCurrent.
	  II.30) Try addBefore to put in the one item.
	  II.32) See if isCurrent true after addBefore
	  II.34) See if size works after addBefore.
	  II.36) See if lookup works addBefore.
 	  II.38) See if getCurrent works after addBefore.
	  II.40) See if back works after addBefore and start
	  II.42) See if isCurrent false after addAfter and back
	  II.44) See if advance works after addBefore and start
	  II.46) See if isCurrent false after addBefore and advance
	  II.48) Try addAfter to put in the one item.
 	  II.50) See if isCurrent true after addAfter
	  II.52) See if size works after addAfter.
	  II.54) See if advance works.
	  II.56) See if isCurrent true after advance
	  II.58) See if back works. 
	  II.60) See if isCurrent true after back
	  II.62) See if removeCurrent works on size 2 Sequence.
	  II.64) See if isCurrent true after removeCurrent
	  II.66) See if getCurrent works after addAfter.
	  II.68) Try addBefore to put in the one item.
	  II.70) See if isCurrent true after addBefore
	  II.72) See if size works after addBefore.
	  II.74) See if removeCurrent works on size 2 Sequence last element.
	  II.76) See if isCurrent false after remove the last element
	  II.78) See if getCurrent throws exception on size 2 Sequence
	  
	III) Test larger number of items
	  III.2) Do a lot of addAfters.
	  III.4) see if items there
	  III.6) See if isCurrent false since should be out of items
	  III.8) See if getCurrent throws exception since current should be out of items
	  III.10) See if size works after do a lot of addAfters of String items.
	  III.12) See if lookup works after do a lot of addAftersof String items.
	  III.14) See if back works after do a lot of addAfters of String items. 
	  III.16) See if isCurrent false after do a lot of addAftersof String items	
	  III.18) See if getCurrent throws exception after do a lot of 
	  III.20) See if removeCurrent throws exception after do a lot of 
                     addAfters of String items after start and back
	  III.22) Do a lot of addBefore in the middle of the sequence
	  III.24) see if items there
	  III.26) See if size works after addBefore 10 items.
	  III.28) See if removeCurrent works.
	  III.30) See if isCurrent true after remove 10 items
	  III.32) See if size works after removeCurrent 10 times.
	  III.34) Try removeCurrent at the end element
	  III.36) See if isCurrent false after remove the last element
	  III.38) See if removeCurrent throws exception when current is undefined
	  III.40) See if getCurrent throws exception on large no. items Sequence
	IV) Test a really big number of items with Integer
	  IV.2) Do a lot of addBefores.
	  IV.4) see if items there
	  IV.6) See if back works after do a lot of addBefores of integer items.
	  IV.8) See if isCurrent false after do a lot of addBefores of integer items
	           after start and back
	  IV.10) See if size works after do a lot of addBefores of integer items.
	  IV.12) See if lookup works after do a lot of addBefores of integer items.
   	  IV.14) Try addAfter 26 items to last element
	  IV.16) see if items there
	  IV.18) See if size works after addAfters 26 items.
	  IV.20) Try removeCurrent 26 items in the middle of the sequence
	  IV.22) See if isCurrent true after remove 26 items in the middle
	  IV.24) See if size works after removeCurrent 26 items.
	  IV.26) See if advance throws exception when go off the sequence
	  IV.28) See if isCurrent false after advance in last element
 */
 	
 /*
 	Problems found with the public interface description of the Sequence class.
 	1. getCurrent on empty Sequence threw ArrayIndexOutOfBoundsException
 	2. back on size 1 Sequence (after addAfter) did not throw an exception as expected
  	3. advance on size 1 Sequence (after start) did not throw an exception as expected
  	4. isCurrent on size 1 Sequence (after removeCurrent) did not return false
  	5. removeCurrent on size 1 Sequence (current undefined) threw exception
  	    java.lang.ArrayIndexOutOfBoundsException
  	6. back on size 1 Sequence (after addBefore and start) did not throw and exception as 
  	    expected
        7. advance on size 1 Sequence (after addBefore and start) did not throw and exception 
            as expected
        8 . isCurrent on size 2 Sequence (after remove the last element) did not return false,
             similiar to problem #4
  	9. getCurrent on size 2 Sequence (after remove the last element) did not throw and 
   	      exception as expected
  	10. isCurrent on large no. items Sequence (after remove the last element) did not 
  	      return false, similiar to problem #4   
  	11. removeCurrent on large no. items Sequence (current undefined) threw exception
              java.lang.ArrayIndexOutOfBoundsException, similiar to problem #5     
  	12. getCurrent on large no. items Sequence (after remove the last element) did not 
  	      throw an exception as expected, similiar to problem #9
  	13. iterate over series of 1000 items and see if have expected value (addBefore) 
  	      but 25 item should have been 25 but got 24.     
  */

public class TestSequence {

	// String for start of normal output
	private static final String BEGIN_OUTPUT = new String (
	  "Sequence test: ");
	// String for start of error output
   	private static final String BEGIN_ERROR = new String (
   	  "***Sequence test: ");
   	// Size when do really big test of Integer obhects
   	private static int BIG_SIZE = 1000;
		      

	/**
	 * main method runs a series of tests on the Sequence class to make sure
	 * it is implemented correctly.
	 * @param String[] where first argument is name of file to place messages.
	 **/
    public static void main(String[] args) {

		// Create a String with lots of values to use for testing.
		// Make every value different so know unique.
		String items = "QWERTYUIOPASDFGHJKLZXCVBNM";
		// Holds beginning of descriptive string of what test running
		String testing;
		// Holds string to print at end when throw exception
		String except = "threw exception";
		// Holds int return values from methods
		int intReturn;
		// Holds Object return values from methods
		Object objectReturn;
		// Holds string value expected
		String stringExpect;
		// Holds integer value expected
		Integer integerExpect;
		// Sequence we will normally use.
		// Set = null since compiler thinks may not be set.  Not sure why.
		Sequence s1 = null;
		// Holds name of file for output
		String myOutName;
		// Holds PrintWriter for output.
		// Set to null here since Java compiler thinks it may not be set
		// before used due to System.exit later.
		PrintWriter myOut = null;

    	// To make sure the window hangs around under CodeWarrior for Unix you
    	// need to have it wait at the end for input.  If the code throws an
    	// unexpected exception, it might terminate without reaching the code
    	// to wait for input.  As a result, the whole main method is inside
    	// a try/finally clause to make sure it reaches the needed code.
    	//  Also, it is critical you do a close on the open file myOut.
    	try {

			System.out.println(BEGIN_OUTPUT + "begin");
			
			// Even if you keep the window around in CodeWarrior, it only shows
			// the last batch of lines of output.  Since I still don't have a 
			// good fix we'll send most output to a file.  This isn't bad anyway 
			// it leaves a permanent record of the output.  The file name is
			// provided in the first command line argument.
				 	
	 		// check that args is correct and get value
	 		if (args.length != 1) {
	 			// p1 should have sent error messages to .err not .out.
	 			// On some systems, e.g., Unix, messages sent to .err can be
	 			// separated from messages from .out so you can quickly see
	 			// them if they occur.  There are other differences like .err
	 			// is immediately printed whereas .out is sometimes delayed.
	 			System.err.println(BEGIN_ERROR +
	 			  "Expected 1 argument to class TestSequence but received " +
	 		 	   args.length + " arguments");
	 			if (args.length > 0) {
	 				System.err.println("The arguments were: ");
	 				for (int i = 0; i < args.length; i++) {
	 					System.err.println("  argument " + i + " == "
	 					  + args[i]);
	 				}
	 			}
	 			// Makes window hang around in CodeWarrior under Unix
	 			CWHelp.keepWindow();
	 			System.exit(99);
	 		}
	 		myOutName = args[0];
			
			// Let's open the file for output
			try {
				File outFile = new File(myOutName);
				myOut = new PrintWriter(new BufferedWriter(
				  new FileWriter(outFile)));
			} catch (IOException ex) {
				System.err.println(BEGIN_ERROR + "Opening file " + myOutName +
				  " failed so stopping.  The error was:");
				System.err.println(ex);
				// Makes window hang around in CodeWarrior under Unix
	 			CWHelp.keepWindow();
	 			System.exit(99);
	 		}
	 		myOut.println(BEGIN_OUTPUT + "begin");
	 		System.out.println("Any error messages are in the file " +
	 		  myOutName);
		
			// I) Tests on empty Sequence
			
			// I.2) Constructor
			testing = "constructor for Sequence ";
			try {
				s1 = new Sequence();
			} catch (Throwable ex) {
		    	myOut.println(BEGIN_ERROR + testing + except);
		   	    // The printStackTrace method in class Throwable shows you
		   	    // where you were in the code when the exception occured.  What
	   		    // it shows will become even clearer once we study recursion.
	   		    // The argument tell it where to print the result.  By default
	   		    // it goes to System.err.
	   			ex.printStackTrace(myOut);
		   		// The printStackTrace also prints the exception.  You can
		   		// also do:
		   		// System.out.println(ex);
	   			// to print the exception ex (name is ex due to 
	   			// Throwable ex above)
	   			// Note you should normally never catch Throwable but use
		   		// Exception instead.  We do this here because we want to catch
		   		// any error so the code can continue.
			}
		
			// I.4) See if size works.
			testing = "size on emtpy Sequence ";
			try {
				intReturn = s1.size();
		 		if (intReturn != 0) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return 0 but " + intReturn);
		  		}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
	   			ex.printStackTrace(myOut);
			}
				
			// I.6) See if start works.
			testing = "start on empty Sequence ";
			try {
		 	   s1.start();
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
		
			// I.8) See if isCurrent false after start
			testing = "isCurrent on empty Sequence after start ";
			try {
				if (s1.isCurrent()) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return false");
				}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}

			// I.10) See if back works
			testing = "back on empty Sequence after start ";
			try {
				s1.back();
		 	  	myOut.println(BEGIN_ERROR + testing +
		 	  	  "did not throw an exception as expected");
			} catch (NoCurrentException ex) {
				// This is ok and expected
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// I.12) See if isCurrent false after back
			testing = "isCurrent on empty Sequence after back ";
			try {
				if (s1.isCurrent()) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return false");
				}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}

			// I.14) See if getCurrent throws exception on
			// empty Sequence.
			testing = "getCurrent on empty Sequence ";
			try {
		 	  	objectReturn = s1.getCurrent();
		 	  	myOut.println(BEGIN_ERROR + testing +
		 	  	  "did not throw an exception as expected");

			} catch (NoCurrentException ex) {
				// This is ok and expected
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}

			// I.16) See if lookup works, e.g., it isn't found.
			testing = "lookup on emtpy Sequence ";
			try {
		 		if (s1.lookup(items.substring(0,1)))  {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return false");
				}
			} catch (Throwable ex) {
		    	myOut.println(BEGIN_ERROR + testing + except);
	   		 	ex.printStackTrace(myOut);
			}
			
			//I.18) See if removeCurrent throws exception on
			// empty Sequence.
			testing = "removeCurrent on empty Sequence";
			try {
				s1.removeCurrent();
				myOut.println(BEGIN_ERROR + testing +
		 	  	  "did not throw an exception as expected");
		 	
		 	} catch (NoCurrentException ex) {
				// This is ok and expected
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// I.20) See if isCurrent false after removeCurrent
			testing = "isCurrent on empty Sequence after removeCurrent ";
			try {
				if (s1.isCurrent()) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return false");
				}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// I.22) See if advance works
			testing = "start on empty Sequence ";
			try {
		 	   s1.start();
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			
			testing = "advance on empty Sequence after start ";
			try {
				s1.advance();
		 	  	myOut.println(BEGIN_ERROR + testing +
		 	  	  "did not throw an exception as expected");
			} catch (NoCurrentException ex) {
				// This is ok and expected
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// I.24) See if isCurrent false after advance
			testing = "isCurrent on empty Sequence after advance ";
			try {
				if (s1.isCurrent()) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return false");
				}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// Maybe some other tests on empty sequences either before, during
			// or after these?

			// II) Let's create and work on size 1 Sequence
		
			// II.2) Try addAfter to put in the one item.
			testing = "addAfter on empty Sequence ";
			try {
	 		   s1.addAfter(items.substring(0,1));
			} catch (Throwable ex) {
		    	myOut.println(BEGIN_ERROR + testing + except);
	   			ex.printStackTrace(myOut);
			}
			
			// II.4) See if isCurrent true after addAfter
			testing = "isCurrent size 1 Sequence (after addAfter) ";
			try {
				if ( ! (s1.isCurrent()) ) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return true");
				}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// II.6) See if start works after addAfter.
			testing = "start on size 1 Sequence (after addAfter) ";
			try {
		 	   s1.start();
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
						
			// II.8) See if size works after addAfter.
			testing = "size on size 1 Sequence (after addAfter) ";
			try {
				intReturn = s1.size();
		 		if (intReturn != 1) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return 1 but " + intReturn);
		  		}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
	   			ex.printStackTrace(myOut);
			}
			
			// II.10) See if lookup works addAfter.
			testing = "lookup on size 1 Sequence (after addAfter) ";
			try {
		 		if ( !(s1.lookup(items.substring(0,1))) )  {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return true.");
				}
			} catch (Throwable ex) {
		    	myOut.println(BEGIN_ERROR + testing + except);
	   		 	ex.printStackTrace(myOut);
			}
			
						
			// II.12) See if getCurrent works after addAfter.
			testing = "getCurrent on size 1 Sequence (after addAfter) ";
			try {
		 	  	objectReturn = s1.getCurrent();
		 	  	stringExpect = items.substring(0,1);
		 	  	if ( !  stringExpect.equals(objectReturn)  ) {
		 	  		myOut.println(BEGIN_ERROR + testing +
		 	  	         "did not return the expected value");
				}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// II.14) See if back works after addAfter and start
			//           run start to make sure the current point to first element
			//           (the only element)
			testing = "start on size 1 Sequence (after addAfter) ";
			try {
		 	   s1.start();
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
						
			testing = "back on size 1 Sequence after (addAfter and start) ";
			try {
				s1.back();
		 	  	myOut.println(BEGIN_ERROR + testing +
		 	  	  "did not throw an exception as expected");
			} catch (NoCurrentException ex) {
				// This is ok and expected
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
									
			// II.16) See if isCurrent false after addAfter and back
			testing = "isCurrent on size 1 Sequence after (addAfter and back) ";
			try {
				if (s1.isCurrent()) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return false");				  
					  
				}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
		   	}
									
			// II.18) See if advance works after addAfter and start
			//           run start to make sure the current point to first element
			//           (the only element)
			testing = "start on size 1 Sequence (after addAfter) ";
			try {
		 	   s1.start();
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
						
			testing = "advance on size 1 Sequence (after addAfter and start) ";
			try {
				s1.advance();
		 	  	myOut.println(BEGIN_ERROR + testing +
		 	  	  "did not throw an exception as expected");
			} catch (NoCurrentException ex) {
				// This is ok and expected
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// II.20) See if isCurrent false after addAfter and advance
			testing = "isCurrent on size 1 Sequence (after addAfter and advance) ";
			try {
				if (s1.isCurrent()) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return false");
				}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// II.22) See if removeCurrent works on size 1 Sequence.
			//           run start to make sure the current point to first element
			//           (the only element)
			testing = "start on size 1 Sequence ";
			try {
		 	   s1.start();
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			testing = "removeCurrent on size 1 Sequence";
			try {
				s1.removeCurrent();
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// II.24) See if isCurrent false after removeCurrent
			testing = "isCurrent on size 1 Sequence (after removeCurrent) ";
			try {
				if (s1.isCurrent()) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return false");
				}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// II.26) See if removeCurrent throws exception when current is undefined
			testing = "removeCurrent on size 1 Sequence (current undefined) ";
			try {
				s1.removeCurrent();
				myOut.println(BEGIN_ERROR + testing +
		 	  	  "did not throw an exception as expected");
		 	
		 	} catch (NoCurrentException ex) {
				// This is ok and expected
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
						
			// II.28) See if size works after removeCurrent.
			testing = "size on size 1 Sequence (after removeCurrent) ";
			try {
				intReturn = s1.size();
		 		if (intReturn != 0) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return 0 but " + intReturn);
		  		}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
	   			ex.printStackTrace(myOut);
			}
			
			
			
			// II.30) Try addBefore to put in the one item.
			testing = "addBefore on empty Sequence ";
			try {
	 		   s1.addBefore(items.substring(1,2));
			} catch (Throwable ex) {
		    	myOut.println(BEGIN_ERROR + testing + except);
	   			ex.printStackTrace(myOut);
			}
			
			
			// II.32) See if isCurrent true after addBefore
			testing = "isCurrent on size 1 Sequence (after addBefore) ";
			try {
				if ( ! (s1.isCurrent()) ) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return true");
				}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
						
			// II.34) See if size works after addBefore.
			testing = "size on size 1 Sequence (after addBefore) ";
			try {
				intReturn = s1.size();
		 		if (intReturn != 1) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return 1 but " + intReturn);
		  		}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
	   			ex.printStackTrace(myOut);
			}
			
			// II.36) See if lookup works addBefore.
			testing = "lookup on size 1 Sequence (after addBefore) ";
			try {
		 		if ( !(s1.lookup(items.substring(1,2))) )  {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return true.");
				}
			} catch (Throwable ex) {
		    	myOut.println(BEGIN_ERROR + testing + except);
	   		 	ex.printStackTrace(myOut);
			}
			
						
			// II.38) See if getCurrent works after addBefore.
			testing = "getCurrent on size 1 Sequence after addBefore ";
			try {
		 	  	objectReturn = s1.getCurrent();
		 	  	stringExpect = items.substring(1,2);
		 	  	if ( !  stringExpect.equals(objectReturn)  ) {
		 	  		myOut.println(BEGIN_ERROR + testing +
		 	  	         "did not return the expected value");
				}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// II.40) See if back works after addBefore and start
			//           run start to make sure the current point to first element
			//           (the only element)
			testing = "start on size 1 Sequence (after addBefore) ";
			try {
		 	   s1.start();
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			testing = "back on size 1 Sequence (after addBefore and start) ";
			try {
				s1.back();
		 	  	myOut.println(BEGIN_ERROR + testing +
		 	  	  "did not throw an exception as expected");
			} catch (NoCurrentException ex) {
				// This is ok and expected
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
									
			// II.42) See if isCurrent false after addAfter and back
			testing = "isCurrent on size 1 Sequence (after addBefore and back) ";
			try {
				if (s1.isCurrent()) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return false");				  
					  
				}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
		   	}
									
			// II.44) See if advance works after addBefore and start
			//           run start to make sure the current point to first element
			//           (the only element)
			testing = "start on size 1 Sequence (after addBefore) ";
			try {
		 	   s1.start();
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
						
			testing = "advance on size 1 Sequence (after addBefore and start) ";
			try {
				s1.advance();
		 	  	myOut.println(BEGIN_ERROR + testing +
		 	  	  "did not throw an exception as expected");
			} catch (NoCurrentException ex) {
				// This is ok and expected
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// II.46) See if isCurrent false after addBefore and advance
			testing = "isCurrent on size 1 Sequence (after addBefore and advance) ";
			try {
				if (s1.isCurrent()) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return false");
				}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// II.48) Try addAfter to put in the one item.
			testing = "addAfter on size 1 Sequence ";
			try {
	 		   s1.addAfter(items.substring(2,3));
			} catch (Throwable ex) {
		    	myOut.println(BEGIN_ERROR + testing + except);
	   			ex.printStackTrace(myOut);
			}
			
			// II.50) See if isCurrent true after addAfter
			testing = "isCurrent on size 2 Sequence (after addAfter) ";
			try {
				if ( ! (s1.isCurrent()) ) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return true");
				}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// II.52) See if size works after addAfter.
			testing = "size on size 2 Sequence (after addAfter) ";
			try {
				intReturn = s1.size();
		 		if (intReturn != 2) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return 2 but " + intReturn);
		  		}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
	   			ex.printStackTrace(myOut);
			}			
			
			// II.54) See if advance works.
			//           run start to make current to be the first element
			testing = "advance on size 2 Sequence ";
			try {
				s1.start();
				s1.advance();
		 	} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
						
			// II.56) See if isCurrent true after advance
			testing = "isCurrent on size 2 Sequence (after advance) ";
			try {
				if ( ! (s1.isCurrent()) ) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return true");
				}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// II.58) See if back works. 
			testing = "back on size 2 Sequence ";
			try {
				s1.back();
		 	} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// II.60) See if isCurrent true after back
			testing = "isCurrent on size 2 Sequence (after back) ";
			try {
				if ( ! (s1.isCurrent()) ) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return true");
				}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
												
			// II.62) See if removeCurrent works on size 2 Sequence.
			testing = "removeCurrent on size 2 Sequence";
			try {
				s1.removeCurrent();
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// II.64) See if isCurrent true after removeCurrent
			testing = "isCurrent on size 2 Sequence (after removeCurrent) ";
			try {
				if ( ! (s1.isCurrent()) ) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return true");
				}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// II.66) See if getCurrent works after addAfter.
			testing = "getCurrent on size 2 Sequence (after addAfter) ";
			try {
		 	  	objectReturn = s1.getCurrent();
		 	  	stringExpect = items.substring(2,3);
		 	  	if ( !  stringExpect.equals(objectReturn)  ) {
		 	  		myOut.println(BEGIN_ERROR + testing +
		 	  	         "did not return the expected value");
				}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// II.68) Try addBefore to put in the one item.
			testing = "addBefore on size 1 Sequence ";
			try {
	 		   s1.addBefore(items.substring(3,4));
			} catch (Throwable ex) {
		    	myOut.println(BEGIN_ERROR + testing + except);
	   			ex.printStackTrace(myOut);
			}
			
			// II.70) See if isCurrent true after addBefore
			testing = "isCurrent on size 2 Sequence (after addBefore) ";
			try {
				if ( ! (s1.isCurrent()) ) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return true");
				}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// II.72) See if size works after addBefore.
			testing = "size on size 2 Sequence (after addBefore) ";
			try {
				intReturn = s1.size();
		 		if (intReturn != 2) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return 2 but " + intReturn);
		  		}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
	   			ex.printStackTrace(myOut);
			}
			
			// II.74) See if removeCurrent works on size 2 Sequence last element.
			//           run advance to make current the last element			
			testing = "removeCurrent on size 2 Sequence last element";
			try {
				s1.advance();
				s1.removeCurrent();
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// II.76) See if isCurrent false after remove the last element
			testing = "isCurrent on size 2 Sequence (after remove the last element) ";
			try {
				if ( s1.isCurrent() ) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return false");
				}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// II.78) See if getCurrent throws exception on size 2 Sequence
			//           upon removing the last element (i.e. current now is undefined)
			testing = "getCurrent on size 2 Sequence (after remove the last element) ";
			try {
		 	  	objectReturn = s1.getCurrent();
		 	  	myOut.println(BEGIN_ERROR + testing +
		 	  	  "did not throw an exception as expected");

			} catch (NoCurrentException ex) {
				// This is ok and expected
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
									
			// III) Test larger number of items
		
			// create a new Sequence
			testing = "constructor for Sequence ";
			try {
				s1 = new Sequence();
			} catch (Throwable ex) {
	 		   	myOut.println(BEGIN_ERROR + testing + except);
   				ex.printStackTrace(myOut);
			}
			// III.2) Do a lot of addAfters.
			//        Each item is one character from
			//        the String items.
			testing = "addAfter on series of " + items.length() + " items ";
			try {
	    		for (int k = 0; k < items.length(); k++) {
					s1.addAfter(items.substring(k,k+1));
			    }
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
	
			// III.4) see if items there
			testing = "iterate over series of " + items.length() +
			  " items and see if have expected value (addAfter) ";
			try {
				s1.start();
	    		for (int k = 0; k < items.length(); k++) {
	    			objectReturn = s1.getCurrent();
	    			stringExpect = items.substring(k,k+1);
	    			// Note I use the stringExpect on the left since I know it
	    			// isn't null of some bad value
					if ( ! stringExpect.equals(objectReturn)) {
						myOut.println(BEGIN_ERROR + testing);
						myOut.println("  but " + k + " item should have been " +
						  stringExpect + " but got " + objectReturn);
					}
					s1.advance();
		    	}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}

			// III.6) See if isCurrent false since should be out of items
			testing = "isCurrent after iterate past end when " + items.length()
			  + " items ";
			try {
				if (s1.isCurrent()) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return false");
				}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// III.8) See if getCurrent throws exception since current should be
			//          out of items
			testing = "getCurrent after iterate past end when " + items.length()
			  + " items ";
			try {
		 	  	objectReturn = s1.getCurrent();
		 	  	myOut.println(BEGIN_ERROR + testing +
		 	  	  "did not throw an exception as expected");

			} catch (NoCurrentException ex) {
				// This is ok and expected
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// III.10) See if size works after do a lot of addAfters of String items.
			testing = "size after adding " + items.length() + "items (addAfter) ";
			try {
				intReturn = s1.size();
		 		if (intReturn != items. length() ) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return " + items. length() + " but " + intReturn);
		  		}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
	   			ex.printStackTrace(myOut);
			}
			
			// III.12) See if lookup works after do a lot of addAfters of String items.
			testing = "lookup after adding " + items.length() + "items (addAfter) ";
			try {
		 		if ( !(s1.lookup(items.substring(25,26))) )  {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return true.");
				}
			} catch (Throwable ex) {
		    	myOut.println(BEGIN_ERROR + testing + except);
	   		 	ex.printStackTrace(myOut);
			}
			
			// III.14) See if back works after do a lot of addAfters of String items. 
			//            run start to make current to be the first element
			testing = "back after do a lot of addAfters of String items and start ";
			try {
				s1.start();
				s1.back();
		 	} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// III.16) See if isCurrent false after do a lot of addAfters of String items
			//            after start and back
			testing = "isCurrent after do a lot of addAfters of String items" +
			   " (after start and back) ";
			try {
				if ( s1.isCurrent() ) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return false");
				}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// III.18) See if getCurrent throws exception after do a lot of 
			//            addAfters of String items after start and back
			testing = "getCurrent after do a lot of addAfters of" + 
			  " String items (after start and back) ";
			try {
		 	  	objectReturn = s1.getCurrent();
		 	  	myOut.println(BEGIN_ERROR + testing +
		 	  	  "did not throw an exception as expected");

			} catch (NoCurrentException ex) {
				// This is ok and expected
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// III.20) See if removeCurrent throws exception after do a lot of 
			//            addAfters of String items after start and back
			testing = "removeCurrent after do a lot of " + 
			  "addAfters of String items after start and back ";
			try {
				s1.removeCurrent();
				myOut.println(BEGIN_ERROR + testing +
		 	  	  "did not throw an exception as expected");
		 	} catch (NoCurrentException ex) {
				// This is ok and expected
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
									
			// III.22) Do a lot of addBefore in the middle of the sequence
			//            Note they are stored in reverse order
			//	  	Each item is an integer, from 0 to 9
			//            run advance 10 times to make current in the middle of the Seq.
			testing = " addBefore 10 items in the middle of the sequence";
			int n = 0;
			try {
			s1.start();
			for (int i = 0; i < 10; i++) s1.advance();
			for (n = 0; n  < 10; n++) {
					// new Integer(n) creates a new Object with the value n
					s1.addBefore(new Integer(n));
			  }
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + " for  item " + n +
	    		  except);
		   		ex.printStackTrace(myOut);
			}
			
			// III.24) see if items there
			testing = "iterate over series of 10 items and"+ 
			   " see if have expected value (addBefore) ";
			try {
				s1.start();
				// go to the last add element
	    		for (int k = 0; k < 10; k++) s1.advance();
	    		// Now go through Sequence which returns items in reverse order.
	    		for (int k = 9; k >= 0 ; k--) {
	    			objectReturn = s1.getCurrent();
	    			integerExpect = new Integer(k);
	    			// Note I use the integerExpect on the left since I know it
	    			// isn't null of some bad value
					if ( ! integerExpect.equals(objectReturn)) {
						myOut.println(BEGIN_ERROR + testing);
						myOut.println("  but " + k + " item added hould have been " +
						  integerExpect + " but got " + objectReturn);
					}
					s1.advance();
		    	}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// III.26) See if size works after addBefore 10 items.
			testing = "size after addBefore 10 items ";
			try {
				intReturn = s1.size();
		 		if (intReturn != (items.length() + 10 ) ) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return " + (items.length() + 10 )+ " but " + intReturn);
		  		}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
	   			ex.printStackTrace(myOut);
			}
			
			// III.28) See if removeCurrent works.
			testing = "removeCurrent on large no. sequence (remove 10 items) ";
			try {
				s1.start(); //go to the front
				for (int i = 0; i <10 ; i++)  s1.removeCurrent();
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
	   			ex.printStackTrace(myOut);
			}
			
			// III.30) See if isCurrent true after remove 10 items
			testing = "isCurrent on large no. Sequence (after remove 10 items) ";
			try {
				if ( ! (s1.isCurrent()) ) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return true");
				}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// III.32) See if size works after removeCurrent 10 times.
			testing = "size after removeCurrent 10 items ";
			try {
				intReturn = s1.size();
		 		if (intReturn != items.length() ) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return " + (items.length() + 10 )+ " but " + intReturn);
		  		}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
	   			ex.printStackTrace(myOut);
			}
			
			// III.34) Try removeCurrent at the end element
			testing = "removeCurrent at the end element on large no. items sequence ";
			try {
				s1.start(); //go to front
				//go to the end element
				for (int j = 0 ; j < (items.length() - 1); j++) s1.advance();
				s1.removeCurrent();
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// III.36) See if isCurrent false after remove the last element
			testing = "isCurrent on large no. items Sequence " +
			  "(after remove the last element) ";
			try {
				if ( s1.isCurrent() ) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return false");
				}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// III.38) See if removeCurrent throws exception when current is undefined
			testing = "removeCurrent on large no. items Sequence (current undefined) ";
			try {
				s1.removeCurrent();
				myOut.println(BEGIN_ERROR + testing +
		 	  	  "did not throw an exception as expected");
		 	
		 	} catch (NoCurrentException ex) {
				// This is ok and expected
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// III.40) See if getCurrent throws exception on large no. items Sequence
			//           upon removing the last element (i.e. current now is undefined)
			testing = "getCurrent on large no. items Sequence (after remove the " + 
			  "last element) ";
			try {
		 	  	objectReturn = s1.getCurrent();
		 	  	myOut.println(BEGIN_ERROR + testing +
		 	  	  "did not throw an exception as expected");

			} catch (NoCurrentException ex) {
				// This is ok and expected
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// IV) Test really big number of items
			// Also test Integer so not all String
		
			// create a new Sequence
			testing = "constructor for Sequence ";
			try {
				s1 = new Sequence();
			} catch (Throwable ex) {
	 		   	myOut.println(BEGIN_ERROR + testing + except);
   				ex.printStackTrace(myOut);
			}
			// IV.2) Do a lot of addBefores.  Note they are stored in reverse 
			// order.  Also, use Integer here since want to test Object other 
			// than String.
			testing = "addBefore on series of " + BIG_SIZE + " items ";
			int m = 0;
			try {
	    		for (m = 0; m < BIG_SIZE; m++) {
	    			// new Integer(m) creates a new Object with the value m
					s1.addBefore(new Integer(m));
			    }
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + " for " + m + " item " +
	    		  except);
		   		ex.printStackTrace(myOut);
			}

			// IV.4) see if items there
			testing = "iterate over series of " + BIG_SIZE +
			  " items and see if have expected value (addBefore) ";
			try {
				s1.start();
				// go to end of Sequence
	    		for (int k = 1; k < BIG_SIZE; k++) s1.advance();
	    		// Now go backward through Sequence which returns items in order.
	    		for (int k = 0; k < BIG_SIZE; k++) {
	    			objectReturn = s1.getCurrent();
	    			integerExpect = new Integer(k);
	    			// Note I use the integerExpect on the left since I know it
	    			// isn't null of some bad value
					if ( ! integerExpect.equals(objectReturn)) {
						myOut.println(BEGIN_ERROR + testing);
						myOut.println("  but " + k + " item should have been " +
						  integerExpect + " but got " + objectReturn);
					}
					s1.back();
		    	}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// IV.6) See if back works after do a lot of addBefores of integer items. 
			//            run start to make current to be the first element
			testing = "back after do a lot of addBefores of integer items and start ";
			try {
				s1.start();
				s1.back();
		 	} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// IV.8) See if isCurrent false after do a lot of addBefores of integer items
			//            after start and back
			testing = "isCurrent after do a lot of addBefores of integer items" +
			   " (after start and back) ";
			try {
				if ( s1.isCurrent() ) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return false");
				}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// IV.10) See if size works after do a lot of addBefores of integer items.
			testing = "size after adding " + BIG_SIZE + "items (addBefores) ";
			try {
				intReturn = s1.size();
		 		if (intReturn != BIG_SIZE ) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return " + BIG_SIZE + " but " + intReturn);
		  		}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
	   			ex.printStackTrace(myOut);
			}
			
			// IV.12) See if lookup works after do a lot of addBefores of integer items.
			testing = "lookup after adding " + BIG_SIZE + "items (addBefores) ";
			try {
		 		if ( !(s1.lookup(new Integer(997) )) )  {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return true.");
				}
			} catch (Throwable ex) {
		    	myOut.println(BEGIN_ERROR + testing + except);
	   		 	ex.printStackTrace(myOut);
			}
			
			// IV.14) Try addAfter 26 items to last element
			testing = "addAfter " + items.length() + " items on series of " +
			  BIG_SIZE + " items ";
			try {
			  s1.start(); // go to the front
			  for (int j = 0 ; j < ( BIG_SIZE - 1 ) ; j++)  s1.advance() ;	
			  for (int k = 0; k < items.length(); k++) {
					s1.addAfter(items.substring(k,k+1));
			    }
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// IV.16) see if items there
			testing = "iterate over series of " + (items.length() + BIG_SIZE) +
			  " items and see if have expected added value (addAfter) ";
			try {
				s1.start();
	    			for (int j = 0 ; j < BIG_SIZE  ; j++)  s1.advance() ;
	    		for (int k = 0; k < items.length(); k++) {
	    			objectReturn = s1.getCurrent();
	    			stringExpect = items.substring(k,k+1);
	    			// Note I use the stringExpect on the left since I know it
	    			// isn't null of some bad value
					if ( ! stringExpect.equals(objectReturn)) {
						myOut.println(BEGIN_ERROR + testing);
						myOut.println("  but " + k + " item added should have been " +
						  stringExpect + " but got " + objectReturn);
					}
					s1.advance();
		    	}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// IV.18) See if size works after addAfters 26 items.
			testing = "size after addAfter 26 items ";
			try {
				intReturn = s1.size();
		 		if (intReturn != (items.length() + BIG_SIZE ) ) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return " + (items.length() + BIG_SIZE )+ " but " + intReturn);
		  		}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
	   			ex.printStackTrace(myOut);
			}
			
			// IV.20) Try removeCurrent 26 items in the middle of the sequence
			testing = "removeCurrent " + items.length() + " items in the middle " +
			  "of the "+ BIG_SIZE+ "items sequence. ";
			try {
				s1.start(); //go to the front
				for (int j = 0; j < 500; j++) s1.advance() ;
				for (int i = 0; i < items.length() ; i++)  s1.removeCurrent();
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
	   			ex.printStackTrace(myOut);
			} 
			  
			// IV.22) See if isCurrent true after remove 26 items in the middle
			testing = "isCurrent on really big Sequence (after remove 26 items" + 
			  " in the middle)";
			try {
				if ( ! (s1.isCurrent()) ) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return true");
				}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}  
			
			// IV.24) See if size works after removeCurrent 26 items.
			testing = "size after removeCurrent 26 items ";
			try {
				intReturn = s1.size();
		 		if (intReturn != (BIG_SIZE ) ) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return " +  BIG_SIZE + " but " + intReturn);
		  		}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
	   			ex.printStackTrace(myOut);
			}
			
			// IV.26) See if advance throws exception when go off the sequence
			testing = "advance throws exception when go off the sequence ";
			try { 
			  	for ( int k = 0; k < BIG_SIZE ; k++) s1.advance() ; 
				myOut.println(BEGIN_ERROR + testing +
		 	  	  "did not throw an exception as expected");
			} catch (NoCurrentException ex) {
				// This is ok and expected
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
			// IV.28) See if isCurrent false after advance in last element
			testing = "isCurrent on really big Sequence after advance in last element";
			try {
				if (s1.isCurrent()) {
					myOut.println(BEGIN_ERROR + testing +
					  "did not return false");
				}
			} catch (Throwable ex) {
	    		myOut.println(BEGIN_ERROR + testing + except);
		   		ex.printStackTrace(myOut);
			}
			
		} catch (Throwable ex) {
			System.out.println(BEGIN_ERROR +
		   	"had unexpected and uncaught exception in main");
		   	ex.printStackTrace();
			}
		finally {
			myOut.println(BEGIN_OUTPUT + "end");
			myOut.close();
			System.out.println(BEGIN_OUTPUT + "end");
			// Makes window hang around in CodeWarrior under Unix
	 		CWHelp.keepWindow();

  		}
	}
}
