// You should not make any modifications to this class!

/**
 * The <CODE>DoubleNode</CODE> class is used as a storage object for data.
 * It further keeps track of the previous and next DoubleNodes in the sequence
 * with the use of <CODE>prev</CODE> and <CODE>next</CODE> DoubleNode data
 * members, both of which are accessible package-wide.
 **/
class DoubleNode {

    // Note: All data members have package-wide access.

    /**
     * The data this node is keeping track of.
     **/
    Object data;

    /**
     * The node preceeding this DoubleNode.  If there is no such node,
     * <CODE>prev</CODE> should have the value <CODE>null</CODE>.
     * Otherwise, it should be the case that <CODE>prev.next == this</CODE>
     **/
    DoubleNode prev;

    /**
     * The node following this DoubleNode.  If there is no such node,
     * <CODE>next</CODE> should have the value <CODE>null</CODE>.  
     * Otherwise it should be the case that <CODE>next.prev == this</CODE>
     **/
    DoubleNode next;


    /**
     * Creates a new, completely empty DoubleNode.  The data, prev, and next
     * references are all set to <CODE>null</CODE>.
     **/
    public DoubleNode() {
        this(null);
    }


    /**
     * Creates a new DoubleNode which contains the specified data.
     * The prev and next references are set to <CODE>null</CODE>.
     * 
     * @param data the data being stored in this DoubleNode
     **/
    public DoubleNode(Object data) {
        this(data, null, null);
    }


    /**
     * Creates a new DoubleNode which contains the specified data with
     * specified prev and next references.
     *
     * @param data the data being stored in this DoubleNode
     * @param prev the node to be previous to this DoubleNode
     * @param next the node to follow this DoubleNode
     **/
    public DoubleNode(Object data, DoubleNode prev, DoubleNode next) {
        this.data = data;
        this.prev = prev;
        this.next = next;
    }
    
    
    /**
     * Returns a String representation of this DoubleNode.  Returns the String
     * given by the toString() method of the data of this node.
     *
     * @return a String representation of this DoubleNode
     **/
    public String toString() {
        return data.toString();
    }
}
     
