/*******************************************************************************
Main Class File:  PhotoAlbum.java 
File:             LLSequence.java

Author:           	  Data Fok, data@cs.wisc.edu
Collaborators:    N/A

Completion Date:  2/27/2001

Course:           CS367, Spring 2001, Section 3
Assignment:       Programming Assignment 2 

Editor:           CodeWarrior
Compiler:         CodeWarrior (JDK 1.2)
Platform:         Unix (Solaris)
*******************************************************************************/

/**
 * A <CODE>LLSequence</CODE> is a structure which holds many items in order and
 * gives access to the data by maintaining a "current" item.  It is possible to 
 * move both forwards and backwards through the LLSequence.  Moving in the
 * LLSequence simply changes the current item in the LLSequence.  It should be 
 * noted that the current item is undefined only when there are no items in the 
 * LLSequence.  The LLSequence is implemented in such a way that it is circular
 * and can hold an arbitrary number of values.  Furthermore, additions and
 * removals will be done in constant time.
 **/
public class LLSequence {
        
    private int size; //size of the sequence
    
    private DoubleNode current; //the current item
       
    /**
     * Creates a new, initially empty LLSequence.  The current node is 
     * undefined.
     **/
    public LLSequence() {
		current = null;
		size = 0;
    }


    /**
     * Returns the number of items currently in this LLSequence.
     *
     * @return the number of items currently in this LLSequence
     **/
    public int size() {
		return size;
    }


    /**
     * Returns but does not remove the current item in this Sequence.
     * An exception is issued if the Sequence is currently empty.
     *
     * @return the current item in this Sequence
     * @exception NoCurrentException if there are no items in this LLSequence
     **/
    public Object getCurrent() throws NoCurrentException {
	 if (current == null) 	throw new NoCurrentException() ;
	 else return current; 	
    } 


    /**
     * Inserts the specified data after the current element.  Simply adds the 
     * item if the LLSequence is empty.  The newly inserted item is considered 
     * to be the current item after the addition.
     *
     * @param data the item to insert into this LLSequence
     **/
    public void addAfter(Object data) {
		DoubleNode myNode = new DoubleNode(data);
		if (size == 0) { //for empty sequence
			current = myNode; 		//make current be the new node
			current.prev = myNode;	//make current's prev pointer point to the new node 
			current.next = myNode;	//make current's next pointer point to the new node
		}
		else if (size > 0) { //for not empty sequence
			  current.next.prev = myNode;  	//make the prev pointer of the node after current
			  						//point to the new node			  
			  myNode.next = current.next;	//make the next pointer of the new node point to
			  						//the node after current		  
			  current.next = myNode;		//make the next pointer of current point to the new
			  						//node
			  myNode.prev = current;		//make the prev pointer of the new node point to 
			  						//current
			  current = myNode;			//make current be the new node
		}//end if
		size++;  //increase the size of the sequence
    }


    /**
     * Inserts the specified element prior to the current element.  Simply adds 
     * the item if the LLSequence is empty.  The newly inserted item is 
     * considered to be the current item after the addition.
     *
     * @param data the item to inset into this LLSequence
     **/
    public void addBefore(Object data) {
		DoubleNode myNode = new DoubleNode(data);
		if (size == 0) { //for empty sequence
			current = myNode;		//make current be the new node
			current.prev = myNode;	//make current's prev pointer point to the new node 
			current.next = myNode;	//make current's next pointer point to the new node
		}
		else if (size > 0) { //for not empty sequence
			  current.prev.next = myNode;	//make the next pointer of the node before current 
			  						//point to the new node
    			  myNode.prev = current.prev;	//make the prev pointer of the new node point to 
    			  						//the node before current
    			  current.prev = myNode;		//make the prev pointer of current point to the new
    			  						//node
    			  myNode.next = current;		//make the next pointer of the new node point
    			  						//to current
    			  current = myNode;			//make the new node be current	
    		}//end if
    		size ++; //increase the size
    }

    
    /**
     * Moves current to the next item in this LLSequence.
     *
     * @exception NoCurrentException if there are no items in this LLSequence
     **/
    public void advance() throws NoCurrentException {
		if (current.next == null) throw new NoCurrentException() ;
		else 	current = current.next;
    }
    

    /**
     * Advances current in the LLSequence the specified number of times.  It is
     * assumed that the number of times to advance is positive.
     *
     * @param times the number of elements to advance 
     * @exception NoCurrentException if there are no items in this LLSequence
     **/
    public void advance(int times)  throws NoCurrentException {
		for ( int i = 0; i < times ; i++)
			advance();
    }

    
    /**
     * Moves current to the previous item in this LLSequence.
     *
     * @exception NoCurrentException if there are no items in this LLSequence
     **/
    public void back()  throws NoCurrentException {
		if (current.prev == null) throw new NoCurrentException() ;
		else current = current.prev;
    }

    
    /**
     * Moves current back in this LLSequence the specified number of times.  It
     * is assumed that the number of times to go back is a positive value.
     * 
     * @param times the number of times to go back
     * @exception NoCurrentException if there are no items in this LLSequence
     **/
    public void back(int times) throws NoCurrentException {
		for ( int i = 0; i < times ; i++)
			back();
    }


    /**
     * Tests whether the specified item is in this LLSequence.  If the item is
     * in this LLSequence, current is updated to refer to this item, and true is
     * returned.  Otherwise, false is returned and no changes are made to the 
     * current reference.  False is also returned if there are no items in this
     * LLSequence.  Note that all comparisons use the equals method.
     *
     * @param item the item to find
     * @return true if the item is found, false otherwise
     **/
    public boolean lookup(Object item) {
		if (size == 0) return false;
		else {
			for (int j = 0; j < size; j++) {
				if ( current.data.equals(item) ) return true; 
				else current = current.next;
			}// end for
			return false;
		}//end else
    }


    /**
     * Removes and returns the current item in this LLSequence.  The current
     * item in this LLSequence is then updated to be the item following current.
     * If the last item in this LLSequence is removed, current becomes
     * undefined.  If there are no items in this LLSequence, an exception is 
     * issued.
     *
     * @return the former current item in the sequence
     * @exception NoCurrentException if there are no items in this LLSequence
     **/
    public Object removeCurrent() throws NoCurrentException {
		DoubleNode temp; //store the current node for return
		if ( size == 0 ) throw new NoCurrentException();
		else if ( size == 1) { //if there is only one photo
			temp = current; //store the current photo for return
			current = null; //set the current be undefine
			size = 0; //set the size to zero
			return temp;
		}
		else {
			temp = current;  //store the current photo for return
			// make the next pointer of the node before current point to the node
			// after current
			current.prev.next = current.next; 
			// make the prev pointer of the node after current point to the node
			// before current
			current.next.prev =  current.prev;
			// make current point to the node after current
			current = current.next;
			size--; //size decrease by 1
			return temp;
		}// end else
    }
}
