// You should not make any modifications to this class!

/**
 * The <CODE>Photo</CODE> class is used to represent a photos.  Each photo 
 * object contains the name of the file in which the image is stored, a caption
 * (which may be blank), and the date the photo was taken.
 * <P>
 * Normally, accessor methods would be included.  They were not, however, 
 * because we wanted to force you to use the methods provided.
 **/
public class Photo {

    // Note that the data members are private!
    
    /**
     * The file containing the photo's image.
     **/
    private String fileName;

    /**
     * The caption for this photo.
     **/
    private String caption;
    
    /**
     * The date the photo was taken.  The date is in the format:
     * <i>y y y y m m d d</i>, for example, February 28, 2001 would be 
     * represented as 20010228.
     **/
    private int date;


    /**
     * Creates a new photo with specified file name and date.  The caption is
     * initially blank.
     *
     * @param fileName the name of the file containing the image
     * @param date the date the photo was taken
     **/
    public Photo(String fileName, int date) {
        this.fileName = fileName;
        this.date = date;
        caption = "";
    }


    /**
     * Sets the caption of the photo to the one given.
     *
     * @param newCaption the new caption for the photo
     **/
    public void setCaption(String newCaption) {
        caption = newCaption;
    }


    /**
     * Determines whether or not another object is equal to this photo.
     * <P> 
     * The result is <CODE>true</CODE> if and only if the argument is a
     * <CODE>Photo</CODE> object which has the same file name and date values 
     * as this object.
     *
     * @param obj the object to compare with
     * @return <CODE>true</CODE> if the objects are the same; false otherwise
     * @see java.lang.Object#equals
     **/
    public boolean equals(Object obj) {
        // Make sure we actually have a Photo object.
        // This also takes care of the case where the other == null
        if(!(obj instanceof Photo)) {
            return false;
        }

        // Now that we know we have the same type, convert the argument to a 
        // Photo: this avoids casts when we access the fields of the argument.
        Photo other = (Photo) obj;
        
        // Recall: only compare file names and dates (not captions).
        return other.fileName.equals(fileName) && other.date == date;
    }
    
    
    /**
     * Returns a String representation of this photoe.  You should use this 
     * method to print a photo object.
     *
     * @return a String representation of this photo
     **/
    public String toString() {
        return fileName + " (" + date + "): " + "\"" + caption + "\"";
    }
}
