/*******************************************************************************
Main Class File:  MazeSolver.java
File:             MazeSolver.java

Author:           Data Fok, data@cs.wisc.edu
Collaborators:    N/A

Completion Date:  3-26-2001

Course:           CS367, Spring 2001, Section 3
Assignment:       Programming Assignment 3 

Editor:           CodeWarrior
Compiler:         CodeWarrior (JDK 1.2)
Platform:         Unix (Solaris)
*******************************************************************************/
//import maze.*;

import maze.InvalidMazeFileException;
import maze.MazePosition;
import maze.Maze;

/**
 * The class is used to solve a maze, what can be thought of as a retecgular grid. One square in the 
 * maze is designated the start position and one the finish position. Each square in the maze is 
 * either a wall (which may not be moved through) or clear (i.e. empty). A player begins at the 
 * starting position, must stay within the maze at all times, and can only move to squares that are 
 * clear. The allowable moves are north, south, east, and west (i.e. no diagonal moves allowed). The 
 * goal is to solve the maze by either finding a path from the start position to the finish position or 
 * determining that no such path exists. 
 * 
 * Bugs:	none known
 **/
public class MazeSolver {

	// Data members
	Maze myMaze;
	MazePosition current, tmpLoc; //current is the current position in the maze
							//tmpLoc is the tempery position use in goNorth, goWest,
							//goSouth and goEast
	Stack path;  //stack keep trace of the path
	boolean done, hasPath, isValid;  //done indicate if the maze is solved
							  //hasPath indicate if there is a path from start to finish
						 	  //isValid indicate if the file of maze is valid
	
	/**
	 * Constructs a MazeSolver object with the maze initialized to the one 
	 * described in the file filename.
	 *
	 * @param filename the name of the file containing the maze description
	 **/
	public MazeSolver(String filename) {
		setMaze(filename);
	}
	
	/** 
	 * Solves the given maze by either finding a path from the start position
	 * to the finish position or determining that no such path exists.
	 **/
	public void solve() {
		if (isValid) {
			current = myMaze.getStart();
			//as the start must be on path
			//mark current as on path
			current.markOnPath();
			//push current in the path stack
			path.push(current);
			if ( myMaze.isFinish(current) ) {
				done = true;	
				hasPath = true;
			} 
			else if ( goNorth() ) { //try if the maze can be solved from north
				done = true;
				hasPath = true;
			}
			else if ( goWest() ) { //try if the maze can be solved from west
				done = true;
				hasPath = true;
			}
			else if ( goSouth() ) { //try if the maze can be solved from south
				done = true;
				hasPath = true;
			}
			else if ( goEast() ) { //try if the maze can be solved from east
				done = true;
				hasPath = true;
			}
			else { //no path from the start to the finish
				done = true;
				hasPath = false;
				current.markVisited();
				path.pop();
			}
		}
	}
	
	/**
	 * Prints the solution path in order from start to finish as described in
	 * the Programming Assignment 3 documentation.
	 **/
	public void printPath() {
		if (isValid) {
			if ( !done ) {
				//when the maze is not solved
				System.out.println("The current maze has not been solved yet ");
			}
			else if ( !hasPath ) {
				//when there is no path from start to finish
				System.out.println("The current maze has no path from start to finish ");
			}
			else {
				//make a temp stack to "re-order" the path 
				Stack tmpS = new Stack();
				while ( !(path.isEmpty() ) ) 
				tmpS.push(path.pop());
				//print the path and make it the original order
				while ( !(tmpS.isEmpty() ) ) {
					System.out.print( ((MazePosition) tmpS.peek() ).toString() );
					path.push( tmpS.pop() );
				}
			}
			System.out.print("\n");
		}
	}
	

	/**
	 * Sets the current maze to the one described in the file filename.
	 * 
	 * @param filename the file containing the new maze description
	 **/
	public void setMaze(String filename) {
		try{
			myMaze = new Maze(filename);
			path = new Stack();
			done = false;
			isValid = true;
		}
		catch( InvalidMazeFileException ex){
			isValid = false;
			System.out.println("The file " + filename + 
			  " does not contain a valid maze description. \n");
		}
	}
	

	/**
	 * Prints the maze to the console.
	 **/
	public void printMaze() { 
		if (isValid) myMaze.print();
	}
	
	
	
	/**
	 * The method examine all paths that start at the square immediately to the north of 
	 * the current square.
	 *
	 * @return true if there is a path to finish from north; otherwise false
	 **/
	private boolean goNorth() {
		//try to go one square north
		tmpLoc = myMaze.getMazePosition( (current.getRow()-1), current.getColumn() );
		//check only isClear() and not equal to null , but other condition, such as isVisited()
		//or isWall(), because it is a must that if the square is clear, it cannot be visited.
		if ( tmpLoc !=null && tmpLoc.isClear() ) {
			current = tmpLoc; //set the tmpLoc to be current
			current.markOnPath();  //mark current as on path
			path.push(current);	//push current in the path stack
			if (myMaze.isFinish(current) ) { //test if current is the finish
				return true;  
			}
			//try solve the maze by other direction
			else if ( goNorth() ) 
				return true; 
			else if ( goWest() )
				return true;
			else if ( goEast() ) 
				return true;
			else { //when the maze cannot be solved after trying all direction
				//set current as visited
				current.markVisited();
				//pop current out of the path stack
				path.pop();
				//set current back to the original position before goNorth()
				current = 	
				  myMaze.getMazePosition( (current.getRow() +1), current.getColumn() );
				return false; 
			}
		}
		else
			return false;
	}
	
	/**
	 * The method examine all paths that start at the square immediately to the west of 
	 * the current square.
	 *
	 * @return true if there is a path to the finish from west; otherwise false
	 **/	
	private boolean goWest() {
		tmpLoc = myMaze.getMazePosition( current.getRow(), (current.getColumn()-1) );
		if ( tmpLoc !=null && tmpLoc.isClear() ) {
			current = tmpLoc; //set the tmpLoc to be current
			current.markOnPath();  //mark current as on path
			path.push(current);	//push current in the path stack
			if (myMaze.isFinish(current) ) { //test if current is the finish
				return true;  
			}
			//try solve the maze by other direction
			else if ( goWest() ) //try to go west
				return true;
			else if ( goSouth() ) //try to go south
				return true;
			else if ( goNorth() ) //try to go north
				return true;
			else { //when the maze cannot be solved after trying all direction
				//set current as visited
				current.markVisited();
				//pop current out of the path stack
				path.pop();
				//set current back to the original position before goWest()
				current = 	
				  myMaze.getMazePosition( current.getRow(), (current.getColumn()+1) );
				return false;
			}
		}
		else
			return false;
	}
	
	
	/**
	 * The method examine all paths that start at the square immediately to the south of 
	 * the current square.
	 *
	 * @return true if there is a path to finish from south; otherwise false
	 **/
	private boolean goSouth() {
		tmpLoc = myMaze.getMazePosition( (current.getRow()+1), current.getColumn() );
		if ( tmpLoc !=null && tmpLoc.isClear() ) {
		
			current = tmpLoc; //set the tmpLoc to be current
			current.markOnPath();  //mark current as on path
			path.push(current);	//push current in the path stack
			if (myMaze.isFinish(current) ) { //test if current is the finish
				return true;  
			}
			//try solve the maze by other direction
			else if ( goSouth() ) 
				return true;
			else if ( goEast() ) 
				return true;
			else if ( goWest() ) 
				return true;
			else { //when the maze cannot be solved after trying all direction
				//set current as visited
				current.markVisited();
				//pop current out of the path stack
				path.pop();
				//set current back to the original position before goSouth()
				current = 	
				  myMaze.getMazePosition( (current.getRow()-1), current.getColumn() );
				return false;
			}
		}
		else
			return false;
	}
	
	
	/**
	 * The method examine all paths that start at the square immediately to the east of 
	 * the current square.
	 *
	 * @return true if there is a path to finish from east; otherwise false
	 **/
	private boolean goEast(){
		tmpLoc = myMaze.getMazePosition( current.getRow(), (current.getColumn()+1) );
		if ( tmpLoc !=null && tmpLoc.isClear() ) {
			current = tmpLoc; //set the tmpLoc to be current
			current.markOnPath();  //mark current as on path
			path.push(current);	//push current in the path stack
			if (myMaze.isFinish(current) ) { //test if current is the finish
				return true;  
			}
			//try solve the maze by other direction
			else if ( goEast() ) 
				return true;
			else if ( goNorth() ) 
				return true;
			else if ( goSouth() ) 
				return true;
			else { //when the maze cannot be solved after trying all direction
				//set current as visited
				current.markVisited();
				//pop current out of the path stack
				path.pop();
				//set current back to the original position before goEast()
				current = 	
				  myMaze.getMazePosition( current.getRow(), (current.getColumn()-1) );
				return false;
			}
		}
		else
			return false;
	}
}
