/*******************************************************************************
Title:            Binary Search Trees
Files:            BSTNode.java, BST.java, Entry.java, AddressBook.java

Author:           Data Fok, data@cs.wisc.edu
Collaborators:    n/a

Due Date:         Wednesday, April 25, 2001
Completion Date:  4-27-2001

Course:           CS 367, Spring 2001, Section <section>
Assignment:       Programming Assignment 4

Editor:           CodeWarrior
Compiler:         CodeWarrior (JDK 1.2)
Platform:         Unix (Solaris)
*******************************************************************************/
import java.io.*;

/**
 * The AddressBook class will be used to represent address books. Each address book has
 * a title in addition to its entries. Each entry in an address book consists of a name and 
 * an email address. The AddressBook class will also be the main class of the program.
 * It can takes in 2 command-line arguments. The first one is the input file name. The 
 * second one is the output file name. If there is no second command-line argument, output 
 * will go to the console.
 *
 * Bugs: none.
 **/
public class AddressBook {

    // Data members
    private Dictionary entries;  // the entries in the address book
    private String title; //title of the address book
                     
    /**
     * Constructs a new AddressBook object with the given title.
     *
     * @param title a short title for the new address book
     **/
    public AddressBook(String title) {
    	entries = new BST();
    	this.title = title;    
    }
    
    /**
     * Adds the entry to the address book.  If there is already an entry
     * in the address book for the name given, a DuplicateException is
     * thrown.
     *
     * @param name the name for the entry to be added
     * @param email the email address for the entry to be added
     * @throws DuplicateException if there is already an entry with the
     *         name given in the address book
     **/
    public void add(String name, String email) throws DuplicateException {
    	entries.insert( new Entry(name, email) );
    }
    	
    /**
     * Changes the email address for the entry with the given name to the
     * email address given.  If there is no entry in the address book with
     * the name given, an ItemNotFoundException is thrown.
     *
     * @param name the name for the entry whose email address is to be changed
     * @param email the new email address 
     * @throws ItemNotFoundException if there is no entry with the name given
     *         in the address book
     **/
    public void change(String name, String email) throws ItemNotFoundException{
    	Entry tmp = (Entry) entries.lookup( new Entry(name, null) );
    	if (tmp == null)
    		throw new ItemNotFoundException();
    	else
    		tmp.setEmail(email);
    }
    	
    /**
     * Displays the entire contents of the address book, in order, to the
     * PrintWriter given.  The title of address is first given, then the   
     * entries are listed in alphabetical order, one per line.
     *
     * @param p the PrintWriter to which to send the output
     **/
    public void display(PrintWriter p) {
    	p.println("Address book: " + title);
    	entries.printAll(p);
    	p.print("\n");
    }
    	
    /**
     * Displays the entry in the address book with the given name to the
     * PrintWriter given.  If an entry with that name is not in the address 
     * book, no output is sent to the PrintWriter.
     *
     * @param name the name of the entry to display
     * @param p the PrintWriter to which to send the output
     **/
    public void display(String name, PrintWriter p) {
    	Entry tmp = (Entry) entries.lookup( new Entry(name, null) );
    	if (tmp == null) return;
    	else p.println( tmp.toString() );
    }
    
    /**
     * Displays the entries whose names are in the range begin to end 
     * (inclusive), in order, to the PrintWriter given.  
     * If begin > end, then nothing is printed.
     *
     * @param begin the name at the low end of the range to be displayed
     * @param end the name at the high end of the range to be displayed
     * @param p the PrintWriter to which to send the output
     **/
    public void display(String begin, String end, PrintWriter p) {
    	if (begin.compareToIgnoreCase(end) > 0) return;
    	else {
    		entries.printRange(new Entry(begin, null), new Entry(end, null), p);
    		p.print("\n");
    	}
    }

    /**
     * Determines if there is an entry in the address book with the given name.
     *
     * @param name the name to look up
     * @return true if there is an entry with the name given in the address 
     *         book; false otherwise
     **/
    public boolean find(String name) {
    	Entry tmp = (Entry) entries.lookup( new Entry(name, null) );
    	if (tmp == null) return false;
    	else return true;
    }
        
    /**
     * Removes the entry with the given name.  If there is no entry in the 
     * address book with the name given, an ItemNotFoundException is thrown.
     *
     * @param name the name for the entry to be removed
     * @throws ItemNotFoundException if there is no entry with the name given
     *         in the address book
     **/
    public void remove(String name) throws ItemNotFoundException{
    	Entry tmp = (Entry) entries.delete( new Entry(name, null) );
    	if (tmp == null) throw new ItemNotFoundException();
    }

    /**
     * The main method takes the name of a file as a command-line
     * argument and uses this file to test the AddressBook class.  
     **/
    public static void main(String [] args) throws IOException {

        final String NEW           = "new";
        final String ADD           = "addEntry";
        final String REMOVE        = "removeEntry";
        final String CHANGE        = "changeEmail";
        final String FIND          = "findName";
        final String DISPLAY_RANGE = "displayRange";
        final String DISPLAY_ALL   = "displayAll";

        PrintWriter out = null;  // PrintWriter to send output to
        AddressBook aBook = null; //the instance of address book
        
        // Put the whole code around a try/catch/finally block so can keep 
        // window around (this is not strictly necessary if you are not 
        // using the Unix version of CodeWarrior).
        try {

            // Only run the program if there are 1 or 2 command line arguments
            if (args.length < 1 || args.length > 2) {
                System.err.println("usage: java <input file name> || " +
                           "java <input file name> <output file name>");
                return;
            }

            // Create the object to read from a file
            BufferedReader fin = new BufferedReader(new FileReader(args[0]));
        
           // decide where to send the output
            if (args.length == 1) //with no second command-line argument, output to console
            	out = new PrintWriter(System.out);
            else  //else output to the file indicate in second command-line argument
            	out = new PrintWriter(new BufferedWriter(new FileWriter(args[1]))); 
                       
            // Parse every line of the input file
            while(fin.ready()) {
                String line = fin.readLine();

                //create a new address book
                if (line.equals(NEW)) {
                    // get the title for the new address book
                    String title = fin.readLine();
                    aBook = new AddressBook(title);
                }
		
		//add entry to the address book
                else if (line.equals(ADD)) {
                    // get the name and email address
                    String name = fin.readLine();
                    String email = fin.readLine();
                    try {
                    	aBook.add(name, email);
                    }
                    catch (DuplicateException ex) {
                    	out.println("Can't add entry: address book " + aBook.title + 
                    	   " already has an entry for "+ name + "\n");
                    }
                }
		
		//remove entry in the address book
                else if (line.equals(REMOVE)) {
                    // get the name on the entry to remove 
                    String name = fin.readLine();
                    try {
                    	aBook.remove(name);
                    }
                    catch (ItemNotFoundException ex) {
                    	out.println("Can't remove entry: " + name + " is not in the address book " 
                    	   + aBook.title + "\n" );
                    }
                }

                //change the email of an entry in the address book
                else if (line.equals(CHANGE)) {
                // get the name and the new email address
                    String name = fin.readLine();
                    String email = fin.readLine();
                    try {
                    	aBook.change(name, email);
                    	out.print("Updated entry: ");
                    	aBook.display(name, out);
                    	out.print("\n");
                    }
                    catch (ItemNotFoundException ex){
                    	out.println("Can't change entry: " + name + " is not in the address book " 
                    	   + aBook.title + "\n" );
                    }
                }
		
		//find and print an entry
                else if (line.equals(FIND)) {
                    // get the name on the entry to find
                    String name = fin.readLine();
                    
                    if ( aBook.find(name) ) {
                    	out.print("Entry found: ");
                    	aBook.display(name, out);
                    	out.print("\n");
		    }
                    else 
                    	out.println(name + " is not in the address book " + aBook.title + "\n");
                }
		
		//print the entries within a given range
                else if (line.equals(DISPLAY_RANGE)) {
                    // get the beginning and ending values
                    String begin = fin.readLine();
                    String end = fin.readLine();
                    out.println("Address book: " + aBook.title +
                       "    Entries from '"+ begin +"' to '" + end +"'");  
                    aBook.display(begin, end, out); 
                }

                //print all the entries
                else if (line.equals(DISPLAY_ALL)) {
		    aBook.display(out);
		}
                
                // print out any other input  
                else {
                    out.flush();
                    System.err.println("Unknown command: '" + line + "'");
                }

            } // closes while

            // Always make sure you close the input files
            fin.close();

        // Put the catch in and do a stack trace so any uncaught exceptions
        // will be displayed.
        } catch (Throwable ex) {
            out.flush();
            System.err.println("AddressBook had an uncaught exception in main");
            ex.printStackTrace();

        // This keeps the window open whether the try block ended normally or
        // whether an exception was thrown in the try block.
        }  finally {
            if (out != null) out.flush();
            CWHelp.keepWindow();
            if (out != null) out.close();
        }        
    }
}


