/*******************************************************************************
Title:            Binary Search Trees
Files:            BSTNode.java, BST.java, Entry.java, AddressBook.java

Author:           Data Fok, data@cs.wisc.edu
Collaborators:    n/a

Due Date:         Wednesday, April 25, 2001
Completion Date:  4-27-2001

Course:           CS 367, Spring 2001, Section <section>
Assignment:       Programming Assignment 4

Editor:           CodeWarrior
Compiler:         CodeWarrior (JDK 1.2)
Platform:         Unix (Solaris)
*******************************************************************************/
import java.io.*;

public class BST implements Dictionary {
	
	private BSTNode root; //the root of the tree
        private static Comparable DItem = null; //the item being deleted
        private static boolean isExist = false; //true if the item to be deleted is in the BST
         
        //constructor
        public BST() { 
        	root = null; 
        }
      
	//insert
	public void insert(Comparable k) throws DuplicateException {
	  if (root == null) {
       		root = new BSTNode(k, null, null);
    	  }
    	  else insert(root, k);
	}

	private static void insert(BSTNode T, Comparable k) throws DuplicateException {
	  // precondition: T != null
    	  if (k.compareTo(T.key) == 0) throw new DuplicateException();
    	  if (k.compareTo(T.key) < 0) {
          	// add k as left child of T if it doesn't already have one
          	// else insert into T's left subtree
          	if (T.left == null) T.left = new BSTNode(k, null, null);
          	else insert(T.left, k);
    	  }
    	  else {
        	// here when k > T's key
        	// insert k as right child of T if it doesn't already have one
        	// else insert into T's right subtree
        	if (T.right == null) T.right = new BSTNode(k, null, null);
        	else insert(T.right, k);
    	  }
	}
	
	//delete
        public Comparable delete(Comparable k) {
        	root = delete(root, k);
        	if (isExist) 
        		return DItem;
        	else 
        		return null;
        }	
              
	private static BSTNode delete(BSTNode T, Comparable k) {
         if (T == null) return null;
         if (k.compareTo(T.key) == 0) {
            
            // T is the node to be removed
            DItem = T.key;  //set T.key as DItem for return use
            isExist = true;  
            if (T.left == null && T.right == null) return null;
            if (T.left == null) return T.right;
            if (T.right == null) return T.left;
            
            // here if T has 2 chidren
            BSTNode tmp = smallestNode(T.right);
            // copy key field from tmp to T
            T.key = tmp.key;

            // now delete tmp from T's right subtree and return
            T.right = delete(T.right, tmp.key);
            return T;
            
         }
         else if (k.compareTo(T.key) < 0) {
             T.left = delete(T.left, k);
             return T;
         }
         else {
             T.right = delete(T.right, k);
             return T;
          }
     	}
     
	private static BSTNode smallestNode(BSTNode T)
     	  // precondition: T is not null
     	  // postcondition: return the node in the subtree rooted at T that
     	  //                        has the smallest value
     	{
           if (T.left == null) return T;
           else return smallestNode(T.left);
     	}
	
	//lookup
        public Object lookup(Comparable k) {
    		return lookup(root, k);
	}

	private static Object lookup(BSTNode T, Comparable k) {
    	  if (T == null) return null;
    	  if (k.compareTo(T.key) == 0) return T.key;
    	  if (k.compareTo(T.key) < 0) {
        	// k < this node's key; look in left subtree
        	return lookup(T.left, k);
    	  }
    	  else {
        	// k > this node's key; look in right subtree
        	return lookup(T.right, k);
    	  }
	}

        //printAll
        public void printAll(PrintWriter p) {
            print(root, p); 
        }
        
        private void print(BSTNode n, PrintWriter p) {
            // if n is null, do nothing
            if (n == null) return;

            // if n is a leaf
            if (n.left == null && n.right == null)
        	p.println(n.key.toString());  // print n

            else {
                print(n.left, p);
                p.println(n.key.toString());
                print(n.right, p);
            }
        }
	
	//printRange
	public void printRange(Comparable begin, Comparable end, PrintWriter p) {
		print(root, begin, end, p); 
	}
	
	private void print(BSTNode n, Comparable begin, Comparable end, PrintWriter p) {
            // if n is null do nothing
            if (n == null)  return;
                    
            // if n i snot within the print range, go to the correct subtree contain the print range
            if (n.key.compareTo(begin) < 0)
              	print(n.right, begin, end, p);
            else if (n.key.compareTo(end) > 0)
            	print(n.left, begin, end, p); 
            else {
                print(n.left, begin, end, p);
                p.println(n.key.toString());
                print(n.right, begin, end, p);
            }
        } 
}
