public interface Dictionary {

    /**
     * Determines if the given item is in the dictionary and returns it
     * if present.
     *
     * @param item the key/data for the item desired
     * @return the key/data if the item is found and null if not found
     **/
    public Object lookup(Comparable item);

    /**
     * Adds the given item to the dictionary.  If the item is already in
     * the dictionary, a <code>DuplicateException</code> is thrown.
     *
     * @param item the key/data for the item to add to the dictionary
     * @throws <code>DuplicateException</code> if the item is already in the
     * dictionary
     **/
    public void insert(Comparable item) throws DuplicateException;

    /**
     * Removes and returns the given item from the dictionary.
     *
     * @param item the key/data for the item to remove from the dictionary
     * @return the item removed from the dictionary if it was found, null
     *         otherwise
     **/
    public Comparable delete(Comparable item);

    /**
     * Prints all the key/data items in the specified range in sorted order
     * from smallest to largest to the PrintWriter given.  When an item is 
     * printed the key/data is output by converting it to a String.
     * 
     * @param begin the minimum key/data value.  Only items in the ordered 
     *        list that are greater than or equal to begin (according to the
     *        compareTo method) will be printed.
     * @param end the maximum key/data value.  Only items in the ordered 
     *        list that are less than or equal to end (according to the
     *        compareTo method) will be printed.
     * @param p the PrintWriter to which to send the output
     **/
    public void printRange(Comparable begin, Comparable end, 
			   java.io.PrintWriter p);

    /**
     * Prints all the key/data items in the ordered list in sorted order from 
     * smallest to largest to the PrintWriter given.  When an item is printed 
     * the key/data is output by converting it to a String.
     *
     * @param p the PrintWriter to which to send the output
     **/
    public void printAll(java.io.PrintWriter p);
}
