/*******************************************************************************
File:             ComparisonSort.java

Author:           Data Fok, data@cs.wisc.edu

Completion Date:  5-9-2001

Course:           CS 367, Spring 2001, Section 3
*******************************************************************************/

/**
 *This class implements four different comparison sorts: selection sort,
 *insertion sort, merge sort, and quick sort. It also has a method that runs all
 *four sorts on the same input array and prints out statistics. 
 **/

public class ComparisonSort {
	static int dataMove;
	
	/**
	 *Constructor of ComparisonSort
	 **/
	public ComparisonSort(){}
	
	
        /**
         *Sorts the given array using the selection sort algorithm. Note: after
         *this method finishes the array is in sorted order.
         *
         *@param A the array to sort
         **/
	public static void selectionSort(Comparable[] A) {
		int j, k, minIndex;
      		int N = A.length;
      		
       		for (k = 0; k < N-1; k++) {
          		minIndex = k;
          		for (j = k+1; j < N; j++) {
             			if (A[j].compareTo(A[minIndex]) < 0) {
                 			minIndex = j;
             			}
          		}
			if (minIndex != k){
			    Comparable tmp = A[minIndex];
			    A[minIndex] = A[k];
			    A[k] = tmp;
			    dataMove+=3;
			}
       		}
       	}
       	
        /**
         *Sorts the given array using the insertion sort algorithm. Note: after
         *this method finishes the array is in sorted order.
         *
         *@param A the array to sort
         **/
       	public static void insertionSort(Comparable[] A) {
     		int k, j;
       		Comparable tmp;
       		int N = A.length;
       		
           
       		for (k = 1; k < N; k++) {
          		tmp = A[k];
          		dataMove++;
          		j = k - 1;
          		while ( (j >= 0) && (A[j].compareTo(tmp) > 0) ) {
              			A[j+1] = A[j]; // move one value over one 
						//place to the right
              			dataMove++;
              			j--;
          		}
          	A[j + 1] = tmp; // insert kth value in correct place 
				//relative to previous values
                dataMove++;
       		}
	}

     /**
      *Sorts the given array using the merge sort algorithm. Note: after
      *this method finishes the array is in sorted order.
      *
      *@param A the array to sort
      **/
     public static void mergeSort(Comparable[] A) {
     	mergeAux(A, 0, A.length - 1);
     }
    
	/**
	 *Auxiliary merge sort method to allow recursive call
	 *
	 *@param A the array to sort
	 *@param low the "head" index of the sub array
	 *@param high the "end" index of the sub array
	 **/
	private static void mergeAux(Comparable[] A, int low, int high) {
        	// Base case
        	if (low == high) return; 

        	// Divide array in half
        	int mid = (low + high) / 2;

	        // Recursively sort each half
        	mergeAux(A, low, mid);
	        mergeAux(A, mid + 1, high);
		
        	// Merge the two sorted halves
	        Comparable[] temp = new Comparable[high - low + 1];
        	int left = low;       // index for left half
        	int right = mid + 1;  // index for right half
        	int pos = 0;          // index for temp

        
        	while ((left <= mid) && (right <= high)) {
                // Choose the smaller of the two values at the left
                //      and right indices; 
                // copy the value into the current position in temp;
                // increment either left or right (as appropriate);
                // increment current position in temp

	                if (A[left].compareTo(A[right]) < 0) {
        	                temp[pos] = A[left];
        	                dataMove++;
        	                left++;
        	        }
               	 	else {
                        	temp[pos] = A[right];
                        	dataMove++;
                        	right++;
                	}
                	pos++;
        	}

        	// Post-condition: either the left half or the right
        	// half has run out of values, so copy the remaining 
        	// items in the other half into temp
	        while (left <= mid) {
        	        temp[pos] = A[left]; 
               		dataMove++;
               		left++;
                	pos++;
        	}

        	while (right <= high) {
                	temp[pos] = A[right];  
                	dataMove++;
                	right++;
       	      		pos++;
       		}
		
        	// Copy values from temp back to A
	        System.arraycopy(temp, 0, A, low, temp.length);
			dataMove+=temp.length;
		
		
	}
       	
	/**
     *Sorts the given array using the quick sort algorithm. Note: after
     *this method finishes the array is in sorted order.
     *
     *@param A the array to sort
     **/
	public static void quickSort(Comparable[] A) {
			quickAux(A, 0, A.length-1);
    }
     	
   	/**
   	 *Choosing the pivot by picking the median from the value of the array in the
   	 *index of low, high and (low+high)/2 ,and put the smallest of the 3 values 
   	 *in A[low], put the largest of the 3 values in A[high], and put the pivot 
   	 *in A[high-1]. 
   	 *
   	 *@param A the array to find the pivot
   	 *@param low the "head" of the array
   	 *@param high the "end" of the array
   	 **/ 
    private static int medianOfThree(Comparable[] A, int low, int high) {
     		
     		//create a new array for 3 
     		//choose pivot by insertionSort 
			//(ignoring the number of dataMove here)
     		Comparable[] tempArr = new Comparable[3];  
     		tempArr[0] = A[low];
     		tempArr[1] = A[(low+high)/2];
     		tempArr[2] = A[high];
			int tmp = dataMove; //store the current dataMove, 
				    			//preventing it disturbed by calling
     				    		//insertionSort method
     		insertionSort(tempArr);
     		
     		//move smallest of 3 values to A[low], 
			//largest to A[high], pivot to A[high-1]
     		A[low] = tempArr[0];
     		A[high] = tempArr[2];
     		A[(low+high)/2] = A[high-1];
     		A[high-1] = tempArr[1];     		
			dataMove = tmp + 4;
     		return ((SortObject)tempArr[1]).getData();
    }     		
	
	/**
	 *Use two "pointers" (indexes) left and right. They start at opposite ends 
	 *of the array and move toward each other until left "points" to an item 
	 *that is greater than the pivot (so it doesn't belong in the left part of 
	 *the array) and right "points" to an item that is smaller than the pivot. 
	 *Those two "out-of-place" items are swapped, and repeat this process 
	 *until left and right cross
	 *
	 *@param A the array to do the partition
   	 *@param low the "head" of the array
   	 *@param high the "end" of the array
   	 **/ 
	private static int partition(Comparable[] A, int low, int high) {
     	// precondition: A.length >= 3

        	int pivot = medianOfThree(A, low, high); 
        	int left = low+1; 
        	int right = high-2;
        	while ( left <= right ) {
             		while (A[left].compareTo(new SortObject(pivot)) < 0) 
			    left++;
             		while (A[right].compareTo(new SortObject(pivot)) > 0) 
			    right--;
             		if (left <= right) {
               			Comparable temp1 = A[left];
               			A[left] = A[right];
               			A[right] = temp1;
               			dataMove+=3;
                 		left++;
                 		right--;
             		}
         	}
         	Comparable temp2 = A[left];
               	A[left] = A[high-1];
               	A[high-1] = temp2;
               	dataMove+=3;
         	return right;
     	}
     	
     /**
	  * The insertionSort used by quick sort when the array partition is less
	  * than 3 items.
	  *
	  *@param A the array to be sorted
   	  *@param low the "head" of the array partition
   	  *@param high the "end" of the array partition
   	  **/ 
     private static void insertionSort(Comparable[] A, int low, int high) {
     		int k, j;
     		Comparable tmp;
     		
     		for (k = (low+1); k <= high; k++) {
     			tmp = A[k];
     			dataMove++;
     			j = k - 1;
     			while ( (j>=low) && (A[j].compareTo(tmp) > 0) ) {
     				A[j+1] = A[j];
     				dataMove++;
     				j--;
     			}
     			A[j+1] = tmp;
     			dataMove++;
     		}
     }

     /**
	 *Auxiliary quick sort method to allow recursive call
	 *
	 *@param A the array to sort
	 *@param low the "head" index of the sub array
	 *@param high the "end" index of the sub array
	 **/
     private static void quickAux(Comparable[] A, int low, int high) {
         	if (high-low < 2) insertionSort(A, low, high);
         	else {
             		int right = partition(A, low, high);
             		quickAux(A, low, right);
             		quickAux(A, right+2, high);
         	}
     }

	/**
	 *Sorts the given array using the four different sorting algorithms and 
	 *prints out statistics. 
	 *The sorts performed are: selection, insertion, merge, and quick. The 
	 *statistics displayed for each sort are: number of comparisons, number of 
	 *data moves, and time (in milliseconds). 
	 *
	 *Note: each sort is given the same array (i.e. in the original order) and 
	 *the input array A is not changed by this method.
	 *
	 *@param A the array to sort
	 **/
	public static void runAllSorts(SortObject[] A) {
		//copy values from array A to four other array for testing
		SortObject[] tempArr1 = new SortObject[A.length];
		SortObject[] tempArr2 = new SortObject[A.length];
		SortObject[] tempArr3 = new SortObject[A.length];
		SortObject[] tempArr4 = new SortObject[A.length];
		System.arraycopy(A, 0, tempArr1, 0 , A.length); 
		System.arraycopy(A, 0, tempArr2, 0 , A.length); 
		System.arraycopy(A, 0, tempArr3, 0 , A.length); 
		System.arraycopy(A, 0, tempArr4, 0 , A.length); 
		
		//statistics varables for each sort
		int comp, dataM;  //compares and  data moves 
		long time, timeB; //sorting time and the current time 
				  //before the sort
		
		//printing output
		System.out.println( Format.leftAlign(14, "Sort") + 
			Format.rightAlign(8, "compares") +
			Format.rightAlign(12, "data moves") + 
			Format.rightAlign(12, "millisecs") );
		
		//gathering statistic for each sort
		
		//running selection sort
		dataMove = 0;
      		SortObject.resetCompares();
		timeB = System.currentTimeMillis();
		selectionSort(tempArr1);
		comp = SortObject.getCompares();
		dataM=dataMove;
		time = System.currentTimeMillis() - timeB;
		printing("selection", comp, dataM, time);
		//check if the sorting work correctly
	       	checkArr(tempArr1);
		
		//running insertion sort
		dataMove = 0;
       		SortObject.resetCompares();
		timeB = System.currentTimeMillis();
		insertionSort(tempArr2);
		comp = SortObject.getCompares();
		dataM=dataMove;
		time = System.currentTimeMillis() - timeB;
		printing("insertion", comp, dataM, time);
		//check if the sorting work correctly
	       	checkArr(tempArr2);
		
		//running merge sort
		dataMove = 0;
        	SortObject.resetCompares();
		timeB = System.currentTimeMillis();
		mergeSort(tempArr3);
		comp = SortObject.getCompares();
		dataM=dataMove;
		time = System.currentTimeMillis() - timeB;
		printing("merge", comp, dataM, time);
		//check if the sorting work correctly
		checkArr(tempArr3);
		
		//running quick sort
		dataMove = 0;
         	SortObject.resetCompares();
		timeB = System.currentTimeMillis();
		quickSort(tempArr4);
		comp = SortObject.getCompares();
		dataM=dataMove;
		time = System.currentTimeMillis() - timeB;
		printing("quick", comp, dataM, time);
		//check if the sorting work correctly
		checkArr(tempArr4);
	
	}
	
	/**
	 * Print the statistics of each sort in the format as given in the sample
	 * output.
	 *
	 * @param prompt the name of the sort
	 * @param comp # of compares of the sort
	 * @param dataM # of data moves of the sort
	 * @param time the runtime of the sort
	 **/
	private static void printing(String prompt, int comp, int dataM, 
				     long time) {
		System.out.println( Format.leftAlign(14, prompt) +
			Format.rightAlign(8, comp) + 
			Format.rightAlign(12, dataM) + 
			Format.rightAlign(12, time) );
		
	}
	
    /**
	 * Print the whole array.
	 *
	 * @param A the array to be printed
	 **/
    private static void printArr(Comparable[] A) {
		for (int i = 0; i<A.length; i++) {
			System.out.print( A[i].toString() + ", ");
		}
		System.out.println("\n");
	}
	
	/**
	 * Check if the sorted array is in ascending order.
	 *
	 * @param A the array to be checked
	 **/
	private static void checkArr(Comparable[] A) {
	    for (int i = 0; i<A.length-1; i++)
		if ( A[i].compareTo(A[i+1]) > 0 )
		    System.out.print("The array is not in ascending order");
	}
}
